@@ -8,6 +8,8 @@ use crate::models::common::{Cost, Distance, Duration, Profile, Timestamp};
 use crate::models::problem::{ActivityCost, Actor, Job, Single, TransportCost};
 use crate::models::solution::Activity;
 use crate::models::OP_START_MSG;
+use crate::utils::compare_floats;
+use std::cmp::Ordering;
 use std::slice::Iter;
 use std::sync::Arc;
 
@@ -36,26 +38,26 @@ impl ConstraintModule for TransportConstraintModule {
         Self::update_route_states(ctx, self.transport.as_ref());
         // NOTE Rescheduling during the insertion process makes sense only if the traveling limit
         // is set (for duration limit, not for distance).
-        if has_travel_limits(&self.limit_func, ctx) {
-            Self::reschedule_departure(ctx, self.transport.as_ref(), false)
+        match (self.limit_func)(&ctx.route.actor) {
+            (None, None) => {}
+            (_, limit_duration) => {
+                Self::advance_departure_time(ctx, self.transport.as_ref(), false);
+                if let Some(limit_duration) = limit_duration {
+                    ctx.state_mut().put_route_state(LIMIT_DURATION_KEY, limit_duration);
+                }
+            }
         }
+
         Self::update_statistics(ctx, self.transport.as_ref());
     }
 
     fn accept_solution_state(&self, ctx: &mut SolutionContext) {
-        ctx.routes.iter_mut().for_each(|route_ctx| {
+        ctx.routes.iter_mut().filter(|route_ctx| route_ctx.is_stale()).for_each(|route_ctx| {
             let transport = self.transport.as_ref();
 
-            if route_ctx.is_stale() {
-                Self::update_route_schedules(route_ctx, transport);
-                Self::update_route_states(route_ctx, transport);
-            }
-
-            Self::reschedule_departure(route_ctx, transport, false);
-
-            if route_ctx.is_stale() {
-                Self::update_statistics(route_ctx, self.transport.as_ref());
-            }
+            Self::update_route_schedules(route_ctx, transport);
+            Self::update_route_states(route_ctx, transport);
+            Self::update_statistics(route_ctx, transport);
         })
     }
 
@@ -79,7 +81,13 @@ impl TransportConstraintModule {
         duration_code: i32,
     ) -> Self {
         Self {
-            state_keys: vec![LATEST_ARRIVAL_KEY, WAITING_KEY],
+            state_keys: vec![
+                LATEST_ARRIVAL_KEY,
+                WAITING_KEY,
+                TOTAL_DISTANCE_KEY,
+                TOTAL_DURATION_KEY,
+                LIMIT_DURATION_KEY,
+            ],
             constraints: vec![
                 ConstraintVariant::HardRoute(Arc::new(TimeHardRouteConstraint { code: time_window_code })),
                 ConstraintVariant::SoftRoute(Arc::new(RouteCostSoftRouteConstraint {})),
@@ -103,11 +111,6 @@ impl TransportConstraintModule {
         }
     }
 
-    /// Optimizes departure time of route.
-    pub(crate) fn optimize_departure_time(route_ctx: &mut RouteContext, transport: &(dyn TransportCost + Send + Sync)) {
-        Self::reschedule_departure(route_ctx, transport, true);
-    }
-
     pub(crate) fn update_route_schedules(ctx: &mut RouteContext, transport: &(dyn TransportCost + Send + Sync)) {
         let (init, actor) = {
             let start = ctx.route.tour.start().unwrap();
@@ -158,19 +161,6 @@ impl TransportConstraintModule {
         });
     }
 
-    pub(crate) fn reschedule_departure(
-        ctx: &mut RouteContext,
-        transport: &(dyn TransportCost + Send + Sync),
-        optimize_whole_tour: bool,
-    ) {
-        if let Some(new_departure_time) = try_get_new_departure_time(transport, ctx, optimize_whole_tour) {
-            let mut start = ctx.route_mut().tour.get_mut(0).unwrap();
-            start.schedule.departure = new_departure_time;
-            Self::update_route_schedules(ctx, transport);
-            Self::update_route_states(ctx, transport);
-        }
-    }
-
     pub(crate) fn update_statistics(ctx: &mut RouteContext, transport: &(dyn TransportCost + Send + Sync)) {
         let start = ctx.route.tour.start().unwrap();
         let end = ctx.route.tour.end().unwrap();
@@ -188,6 +178,35 @@ impl TransportConstraintModule {
         ctx.state_mut().put_route_state(TOTAL_DISTANCE_KEY, total_dist);
         ctx.state_mut().put_route_state(TOTAL_DURATION_KEY, total_dur);
     }
+
+    /// Tries to move forward route's departure time.
+    pub(crate) fn advance_departure_time(
+        route_ctx: &mut RouteContext,
+        transport: &(dyn TransportCost + Send + Sync),
+        consider_whole_tour: bool,
+    ) {
+        let new_departure_time = try_advance_departure_time(route_ctx, transport, consider_whole_tour);
+        Self::try_update_route_departure(route_ctx, transport, new_departure_time);
+    }
+
+    /// Tries to move backward route's departure time.
+    pub(crate) fn recede_departure_time(route_ctx: &mut RouteContext, transport: &(dyn TransportCost + Send + Sync)) {
+        let new_departure_time = try_recede_departure_time(route_ctx);
+        Self::try_update_route_departure(route_ctx, transport, new_departure_time);
+    }
+
+    fn try_update_route_departure(
+        ctx: &mut RouteContext,
+        transport: &(dyn TransportCost + Send + Sync),
+        new_departure_time: Option<f64>,
+    ) {
+        if let Some(new_departure_time) = new_departure_time {
+            let mut start = ctx.route_mut().tour.get_mut(0).unwrap();
+            start.schedule.departure = new_departure_time;
+            Self::update_route_schedules(ctx, transport);
+            Self::update_route_states(ctx, transport);
+        }
+    }
 }
 
 struct TimeHardRouteConstraint {
@@ -385,10 +404,6 @@ impl TravelHardActivityConstraint {
     }
 }
 
-fn has_travel_limits(limit_func: &TravelLimitFunc, route_ctx: &RouteContext) -> bool {
-    matches!((limit_func)(&route_ctx.route.actor), (Some(_), _) | (_, Some(_)))
-}
-
 /// Applies fixed cost for actor usage.
 struct RouteCostSoftRouteConstraint {}
 
@@ -466,9 +481,9 @@ impl SoftActivityConstraint for CostSoftActivityConstraint {
     }
 }
 
-fn try_get_new_departure_time(
-    transport: &(dyn TransportCost + Send + Sync),
+fn try_advance_departure_time(
     route_ctx: &RouteContext,
+    transport: &(dyn TransportCost + Send + Sync),
     optimize_whole_tour: bool,
 ) -> Option<Timestamp> {
     let first = route_ctx.route.tour.get(1)?;
@@ -509,6 +524,30 @@ fn try_get_new_departure_time(
     }
 }
 
+fn try_recede_departure_time(route_ctx: &RouteContext) -> Option<Timestamp> {
+    let first = route_ctx.route.tour.get(1)?;
+    let start = route_ctx.route.tour.start()?;
+
+    let max_change = *route_ctx.state.get_activity_state::<f64>(LATEST_ARRIVAL_KEY, first)? - first.schedule.arrival;
+
+    let earliest_allowed_departure =
+        route_ctx.route.actor.detail.start.as_ref().and_then(|s| s.time.earliest).unwrap_or(start.place.time.start);
+
+    let max_change = (start.schedule.departure - earliest_allowed_departure).min(max_change);
+
+    let max_change = route_ctx
+        .state
+        .get_route_state::<f64>(TOTAL_DURATION_KEY)
+        .zip(route_ctx.state.get_route_state::<f64>(LIMIT_DURATION_KEY))
+        .map(|(&total, &limit)| (limit - total).min(max_change))
+        .unwrap_or(max_change);
+
+    match compare_floats(max_change, 0.) {
+        Ordering::Greater => Some(start.schedule.departure - max_change),
+        _ => None,
+    }
+}
+
 #[allow(clippy::unnecessary_wraps)]
 fn fail(code: i32) -> Option<ActivityConstraintViolation> {
     Some(ActivityConstraintViolation { code, stopped: true })
